// Control.cpp Copyright 1999 Steve Ludtke
// 1/14/99   This is where most of the logic for 'boxer' resides
#include "Control.h"

#define Inherited ControlData

#include <qprogressdialog.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qfiledlg.h>
#include <qapp.h>
#include <qkeycode.h>

Control::Control(QWidget* parent,const char* name):Inherited( parent, name ) {
	QPopupMenu *m1;

	initMetaObject();
	setCaption( "Boxer" );
	
	mgrp->setExclusive(TRUE);

	// FILE MENU
	m1=new QPopupMenu(NULL,"File");
	menu->insertItem("File",m1,100);
	m1->insertItem("Read Image",this,SLOT(Mreadimage()),0);
	m1->insertItem("Save Boxed Areas",this,SLOT(Msaveclips()),0);
	m1->insertSeparator();
	m1->insertItem("Exit",this,SLOT(Mquit()),ALT+Key_X);

	// PROCESS MENU
	m1=new QPopupMenu(NULL,"Boxes");
	menu->insertItem("Boxes",m1);
	m1->insertItem("Read Boxes",this,SLOT(Mreadbdb()),0);
	m1->insertItem("Save Boxes",this,SLOT(Msavebdb()),0);
	menu->insertSeparator();

	// HELP MENU
	m1=new QPopupMenu(NULL,"Help");
	menu->insertItem("Help",m1);
	m1->insertItem("About boxer",this,SLOT(Mabout()),0);
	

	imview=new Image;
	imview->resize(500,500);
	imview->setScale(1.0);
	imview->show();
	
	clipview=new QScrollView();
	clipview->resize(500,300);
	clipview->viewport()->setPalette(QPalette(black));
	clipview->show();

	ptcl.setAutoDelete(TRUE);
	ptclim.setAutoDelete(TRUE);

	apix=4.6667;
	boxsize=100;
	data=NULL;

	curshape=-1;
	
	connect(measure,SIGNAL(clicked()),this,SLOT(setMeasure()));
	connect(select,SIGNAL(clicked()),this,SLOT(setSelect()));
	connect(zap,SIGNAL(clicked()),this,SLOT(setDelete()));

	connect(Size,SIGNAL(textChanged(const char *)),this,SLOT(setSize(const char *)));
	connect(APix,SIGNAL(textChanged(const char *)),this,SLOT(setSize(const char *)));
	connect(scalesel,SIGNAL(textChanged(const char *)),this,SLOT(setScale(const char *)));

	connect(brite,SIGNAL(valueChanged(int)),this,SLOT(setBC(int)));
	connect(cont,SIGNAL(valueChanged(int)),this,SLOT(setBC(int)));

	connect(imview,SIGNAL(mousePress(QMouseEvent *)),this,SLOT(imDown(QMouseEvent *)));
	connect(imview,SIGNAL(mouseMove(QMouseEvent *)),this,SLOT(imMove(QMouseEvent *)));
	connect(imview,SIGNAL(mouseRelease(QMouseEvent *)),this,SLOT(imUp(QMouseEvent *)));
}


Control::~Control() {
delete data;
delete imview;
delete clipview;

}

void Control::setMeasure() {
mode=MEASURE;
}

void Control::setSelect() {
mode=SELECT;
}

void Control::setDelete() {
mode=DELETE;
}

void Control::setSize(const char *t) {
boxsize=atoi(t);
}

void Control::setAPix(const char *t) {
apix=atof(t);
}

void Control::setScale(const char *t) {
scale=atof(t);
imview->setScale(scale);
}

void Control::setBC(int value) {
float b=(brite->value()-500.0)/500.0;
float c=cont->value()/1000.0;
Image *im;

imview->setBrite(b);
imview->setCont(c);

for (im=ptclim.first(); im!=NULL; im=ptclim.next()) {
	im->setBrite(b);
	im->setCont(c);
}

}

void Control::Mabout() { 
	QMessageBox::about(this,"About Boxer",
 "boxer\nCopyright 1999 Steve Ludtke\n\n\
 Boxer is designed to cut out particle\n\
 images from whole micrographs. It has\n\
 features for both manual and automatic\n\
 particle selection.\n");
}

void Control::readimage(char *fsp) { 
QString s(fsp);
QFile file(s);

QApplication::setOverrideCursor( waitCursor );

if (data) delete data;
data=new EMData;
data->readImage(s.data(),0);
data->normalize();
imview->setData(data);


QApplication::setOverrideCursor( arrowCursor );

}

void Control::Mreadimage() { 
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
QFile file(s);

QApplication::setOverrideCursor( waitCursor );

if (data) delete data;
data=new EMData;
data->readImage(s.data(),0);
data->normalize();
imview->setData(data);


QApplication::setOverrideCursor( arrowCursor );

}

void Control::Msaveclips() {
QString s( QFileDialog::getSaveFileName() ),t;
if ( s.isNull() ) return;
int lx,ly;
EMData *d;
int fno=1;

QApplication::setOverrideCursor( waitCursor );

d=ptcl.first();
lx=d->xSize();
ly=d->ySize();

for (d=ptcl.first(); d!=NULL; d=ptcl.next()) {
	if (d->xSize()!=lx || d->ySize()!=ly) {
		t.sprintf(".%d",fno++);
		if (s[s.length()-4]=='.') s.insert(s.length()-4,t);
		else s.append(t);
	}
	d->writeIMAGIC(s.data(),-1);
	lx=d->xSize();
	ly=d->ySize();
}

QApplication::setOverrideCursor( arrowCursor );

}

void Control::Mreadbdb() {
QString s( QFileDialog::getOpenFileName() );
if ( s.isNull() ) return;
FILE *in=fopen(s.data(),"r");
if (!in) return;

}

void Control::Msavebdb() {
int i;
Shape *s;
QString s( QFileDialog::getSaveFileName() ),t;
if ( s.isNull() ) return;
FILE *out=fopen(s.data(),"w");
if (!out) return;

for (i=0; i<imview->nShapes(); i++) {
	s=imview->shapeAt(i);
	fprintf(out,"%d\t%d\t%d\t%d\n",s->x0,s->y0,s->x1,s->y1);
}

fclose(out);

}

void Control::Mquit() { exit(0); }

void Control::clipClick(QMouseEvent *e) {
int i;
Shape *s;

i=ptclim.find((Image *)sender());
if (i<0) return;

switch (mode) {
case SELECT:
	s=imview->shapeAt(i);
	imview->setOrigin(s->x0-imview->width()/2+s->x1/2,s->y0-imview->height()/2+s->y1/2);
	break;
case DELETE:
	delClip(i);
	break;
}

}

void Control::delClip(int i) {
imview->delShape(i);
ptclim.remove(i);
ptcl.remove(i);
redisplayBoxes(i);
imview->imageChanged();
}

void Control::imDown(QMouseEvent *e) {
int n=imview->nShapes();
int i;
Shape *s;

if (curshape!=-1) printf("Down before up!\n");

mx0=e->x();
my0=e->y();

switch (mode) {
case MEASURE:
	s=new Shape;
	s->type=Shape::LINE;
	s->x1=s->x0=imview->pxtodx(e->x());
	s->y1=s->y0=imview->pytody(e->y());
	s->text=new QString("");
	curshape=imview->addShape(s);
	break;
case SELECT:
	// see if click was inside existing box
	for (i=0; i<n ; i++) {
		s=imview->shapeAt(i);
		if (e->x()>imview->dxtopx(s->x0) && e->x()<imview->dxtopx(s->x0+s->x1) &&
			e->y()<imview->dytopy(s->y0) && e->y()>imview->dytopy(s->y0+s->y1)) {
			curshape=i;
			sx0=s->x0;
			sy0=s->y0;
			break;
		}
	}
	if (i!=n) break;

	// if not, make a new one
	s=new Shape;
	s->type=Shape::DOTBOX;
	sx0=s->x0=imview->pxtodx(e->x())-boxsize/2;
	sy0=s->y0=imview->pytody(e->y())-boxsize/2;
	s->x1=boxsize;
	s->y1=boxsize;
	curshape=imview->addShape(s);
	s->text=new QString("");
	s->text->sprintf("%0d",curshape+1);
	ptcl.append(new EMData());
	ptclim.append(new Image());
	ptclim.at(curshape)->resize(boxsize,boxsize);
	connect(ptclim.at(curshape),SIGNAL(mousePress(QMouseEvent *)),this,SLOT(clipClick(QMouseEvent *)));
	imview->imageChanged();
	clipview->addChild(ptclim.at(curshape));
//printf("select %d %d   %d %d\n",mx0,my0,sx0,sy0);
	break;
case DELETE:
	for (i=0; i<n ; i++) {
		s=imview->shapeAt(i);
		if (e->x()>imview->dxtopx(s->x0) && e->x()<imview->dxtopx(s->x0+s->x1) &&
			e->y()<imview->dytopy(s->y0) && e->y()>imview->dytopy(s->y0+s->y1)) break;
	}
	if (i==n) break;
	delClip(i);
	break;
}

}

void Control::imMove(QMouseEvent *e) {
Shape *s;

if (curshape<0) return;

switch (mode) {
case MEASURE:
	s=imview->shapeAt(curshape);
	s->x1=imview->pxtodx(e->x());
	s->y1=imview->pytody(e->y());
	s->text->sprintf("%d pix %1.2f A",(int)hypot(s->x1-s->x0,s->y1-s->y0),hypot(s->x1-s->x0,s->y1-s->y0)*apix);
	imview->imageChanged();
	break;
case SELECT:
	s=imview->shapeAt(curshape);
	s->x0=(int)floor((e->x()-mx0)/imview->lscale+sx0);
	s->y0=(int)floor(-(e->y()-my0)/imview->lscale+sy0);
	imview->imageChanged();
	break;
case DELETE:

	return;
}

}

void Control::imUp(QMouseEvent *e) {
Shape *s;
EMData *d;

if (curshape<0) return;

switch (mode) {
case MEASURE:
	imview->delShape(curshape);
	curshape=-1;
	break;
case SELECT:
	s=imview->shapeAt(curshape);
	s->x0=(int)floor((e->x()-mx0)/imview->lscale+sx0);
	s->y0=(int)floor(-(e->y()-my0)/imview->lscale+sy0);
	imview->imageChanged();
	d=data->clip(s->x0,s->y0,s->x1,s->y1);
	d->normalize();
	ptcl.remove(curshape);
	ptcl.insert(curshape,d);
	ptclim.at(curshape)->setData(d);
	ptclim.at(curshape)->show();
	redisplayBoxes(curshape);
	curshape=-1;
	break;
case DELETE:
	return;
}


}

void Control::redisplayBoxes(int n) {
Image *im;
Shape *s;
int vw=clipview->contentsRect().width(),vh=clipview->contentsRect().height();
int tw=0,th=0,mh=0,i=1;
int totw=0,toth=0;
int visx=0,visy=0;

for (im=ptclim.first(),i=0; im; im=ptclim.next(),i++) {
	clipview->moveChild(im,tw,th);
	if (i==n) { visx=tw; visy=th; }
	tw+=im->getData()->xSize()+3;
	if (im->getData()->ySize()>mh) mh=im->getData()->ySize();
	if (tw+im->getData()->xSize()>vw) { 
		if (tw>totw) totw=tw;
		th+=mh+3; 
		tw=0; 
		mh=0; 
	}
}
toth=th+mh;

clipview->resizeContents(totw,toth);
clipview->ensureVisible(visx,visy);

for (i=0; i<imview->nShapes(); i++) {
	s=imview->shapeAt(i);
	s->text->sprintf("%0d",i+1);
}

}
