#include "PyCylinder.h"
#include "PyEMData.h"
#include <boost/python/detail/extension_class.hpp>

using std::vector;

PyCylinder::PyCylinder() : Cylinder() { }

PyCylinder::PyCylinder(const PyVect& center, const PyVect& axisdir,
		       float len, float r, float w)
    : Cylinder(center, axisdir, len, r, w) { }

PyVect PyCylinder::py_Center()
{
    Vect v = Center();
    return *(static_cast<PyVect*>(&v));
}

PyVect PyCylinder::py_Axis()
{
    Vect v = Axis();
    return *(static_cast<PyVect*>(&v));
}

PyQuater PyCylinder::py_QuaterForAxis()
{
    Quater q = QuaterForAxis();
    return *(static_cast<PyQuater*>(&q));    
}

float PyCylinder::py_PlaneAngleDiff(PyCylinder_ptr next)
{
    return PlaneAngleDiff(next.get());    
}

float PyCylinder::py_AxisAngleDiff(PyCylinder_ptr next)
{
    return AxisAngleDiff(next.get());    
}

float PyCylinder::py_Angle2Center(PyCylinder_ptr next)
{
    return Angle2Center(next.get());    
}

float PyCylinder::py_NormDistance(PyCylinder_ptr next) 
{
    return NormDistance(next.get());    
}

float PyCylinder::py_CenterDistance(PyCylinder_ptr next) 
{
    return CenterDistance(next.get());    
}

bool PyCylinder::py_if_merge(PyCylinder_ptr next, float da_limit,
			     float distance_limit) 
{
    return if_merge(next.get(), da_limit, distance_limit);     
}

bool PyCylinder::py_merge(PyCylinder_ptr next)
{
    return merge(next.get());    
}



// class PyCylinders

PyCylinders PyCylinders::py_copy()
{
    Cylinders c = copy();
    return *(static_cast<PyCylinders*>(&c));
}

PyCylinders PyCylinders::py_subset(PyList data_list, int n)
{
#ifdef DEBUG
    assert(data_list.size() >= n);
#endif
    int* data_array = new int[data_list.size()];
    list2array<int>(data_list, data_array);
    Cylinders c = subset(data_array, n);
    array2list<int>(data_array, data_list);
    
    delete [] data_array;
    data_array = 0;
    
    return *(static_cast<PyCylinders*>(&c));
}

bool PyCylinders::py_add(PyCylinder_ptr cyl)    
{
    return add(cyl.get());
}

bool PyCylinders::py_ReadListfile(const char * in)
{
    //return ReadListfile((char*) in);
    return ReadListfile(const_cast<char*>(in));
}

bool PyCylinders::py_WriteListfile(const char * out)
{
    return WriteListfile(const_cast<char*>(out));
}

bool PyCylinders::py_ReadInventor(const char * in)
{
    return ReadInventor(const_cast<char*>(in));
}

bool PyCylinders::py_WriteInventor(const char * out)
{
    return WriteInventor(const_cast<char*>(out));
}

bool PyCylinders::py_WriteVRML(const char *out)
{
    return WriteVRML(const_cast<char*>(out));
}

bool PyCylinders::py_ReadDejaVu(const char * in)
{
    return ReadDejaVu(const_cast<char*>(in));
}

bool PyCylinders::py_ReadCosec(const char * in)
{
    return ReadCosec(const_cast<char*>(in));
}


PyCylinder_ptr PyCylinders::get(int i)
{
    Cylinder* c = (*this)[i];
    PyCylinder* py_c = static_cast<PyCylinder*>(c);
    return PyCylinder_ptr(py_c);
}


// class PyObjectStat

BOOST_PYTHON_BEGIN_CONVERSION_NAMESPACE
static python_extension_class_converters<PyVect> _PyVect_type;
BOOST_PYTHON_END_CONVERSION_NAMESPACE

void PyObjectStat::py_SetObject(PyEMData_ptr map, const PyList& obj_list)
{
    vector<Point*> vect;
    for (int i = 0; i < obj_list.size(); i++) {
	PyObject* obj = obj_list.get_item(i).get();
	PyVect* item = BOOST_PYTHON_CONVERSION::from_python(obj, boost::python::type<PyVect*>());
	vect.push_back(item);
    }
    SetObject(map.get(), vect);
}

void PyObjectStat::py_SetObject(PyEMData_ptr map)
{
    SetObject(map.get());
}


void PyObjectStat::py_EigenSortTo(PyObjectStat_ptr obj)
{
    EigenSortTo(obj.get());
}

