#ifndef _cylinder_h_
#define _cylinder_h_

#include "PyList.h"
#include "Cylinder.h"
#include "PyVect.h"
#include <boost/smart_ptr.hpp>


class PyCylinder;
typedef boost::shared_ptr<PyCylinder> PyCylinder_ptr;

class PyEMData;
typedef boost::shared_ptr<PyEMData> PyEMData_ptr;

class PyCylinder : public Cylinder {
public:
    PyCylinder();
    PyCylinder(const PyVect& center, const PyVect& axisdir,
	       float len = 5.4, float r = 2.5, float w = 0.9);
    
    PyVect py_Center();
    PyVect py_Axis();
    PyQuater py_QuaterForAxis();
    float py_PlaneAngleDiff(PyCylinder_ptr next);
    float py_AxisAngleDiff(PyCylinder_ptr next);
    float py_Angle2Center(PyCylinder_ptr next);
    float py_NormDistance(PyCylinder_ptr next); 
    float py_CenterDistance(PyCylinder_ptr next); 
    bool py_if_merge(PyCylinder_ptr next, float da_limit = 30. * PI / 180.,
		     float distance_limit = 3.2); 
    bool py_merge(PyCylinder_ptr next);
};


class PyCylinders : public Cylinders {
public:
    
    PyCylinders py_copy();
    PyCylinders py_subset(PyList data_list, int n);

    bool py_add(PyCylinder_ptr cyl);    

    bool py_ReadListfile(const char * in = "-");
    bool py_WriteListfile(const char * out = "-");

    bool py_ReadInventor(const char * in = "-");
    bool py_WriteInventor(const char * out = "-");

    bool py_WriteVRML(const char *out = "-");
    bool py_ReadDejaVu(const char * in="-");

    bool py_ReadCosec(const char * in="-");
    
    PyCylinder_ptr get(int i);
};

class PyObjectStat;
typedef boost::shared_ptr<PyObjectStat> PyObjectStat_ptr;

class PyObjectStat : public ObjectStat {
public:
    void py_SetObject(PyEMData_ptr map, const PyList& obj);
    void py_SetObject(PyEMData_ptr map);
    void py_EigenSortTo(PyObjectStat_ptr obj);
};


#endif
