#include "PyEMData.h"
#include <boost/python/detail/extension_class.hpp>
#include <boost/python/reference.hpp>

#ifdef DEBUG
#include "assert.h"
#endif

static float* get_float_ptr(Float_ptr f)
{
    float* pf = 0;
    Float* ff = f.get();
    
    if (ff) {
	pf = &ff->val;
    }
    return pf;
}

BOOST_PYTHON_BEGIN_CONVERSION_NAMESPACE
static python_extension_class_converters<PyEMData> _PyEMData_type;
BOOST_PYTHON_END_CONVERSION_NAMESPACE

void pylist2emlist(const PyList& pylist, List* emlist)
{
    if (!emlist) {
	return;
    }
    
#ifdef DEBUG
    assert(emlist->count() == 0);
#endif
    
    for (int i = 0; i < pylist.size(); i++) {
	PyObject* obj = pylist.get_item(i).get();
	PyEMData* item = BOOST_PYTHON_CONVERSION::from_python(obj, boost::python::type<PyEMData*>());
	emlist->addObject(item);
    }
    
}

void emlist2pylist(List* emlist, PyList& pylist)
{    
    if (!emlist) {
	return;
    }
    
#ifdef DEBUG
    assert(pylist.size() == 0);
#endif
    
    for (int i = 0; i < emlist->count(); i++) {
	PyEMData item = *(static_cast<PyEMData*>(emlist->objectAt(i)));
	PyObject* obj = BOOST_PYTHON_CONVERSION::to_python(item);
	pylist.append(boost::python::ref(obj));
    }
}





PyEMData::PyEMData() : EMData()
{
    
}


PyEMData::~PyEMData()
{
    
}

int fileCount(const char* filespec, String& filetype)
{
    int type = 0;
    int nimg = fileCount(filespec, &type);
    filetype.val = "unknown";

    if (type == 0) {
        filetype.val = "mrc";
    }
    else if (type == 1) {
        filetype.val = "spi";
    }
    else if (type == 2) {
        filetype.val = "img";
    }
    else if (type == 4) {
        filetype.val = "pgm";
    }
    else if (type == 5) {
        filetype.val = "lst";
    }
    else if (type == 6) {
        filetype.val = "pif";
    }

    return nimg;
}

// return 0 if OK; 
int PyEMData::readImage(const char* filespec, const char* filetype, int n, int nodata)
{
    if (!filespec) {
	printf("Error: no file is input\n");
	return -1;
    }

    if (!filetype) {
	return EMData::readImage(filespec, n, nodata);
    }
    else if (strcmp(filetype, "sal") == 0) {
	return readSAL(filespec, nodata);
    }
    else if (strcmp(filetype, "pgm") == 0) {
	return readPGM(filespec, nodata);
    }
    else if (strcmp(filetype, "icos") == 0) {
	return readICOS(filespec, nodata);
    }
    else if (strcmp(filetype, "mrc") == 0) {
	return readMRC(filespec, nodata);
    }
    else if (strcmp(filetype, "img") == 0) {
	return readIMAGIC(filespec, n, nodata);
    }
    else if (strcmp(filetype, "img3d") == 0) {
	return readIMAGIC3d(filespec, nodata);
    }
    else if (strcmp(filetype, "emim") == 0) {
	return readEMIM(filespec, n, nodata);
    }
    else if (strcmp(filetype, "spi") == 0) {
	return readSPIDER(filespec, n, nodata);
    }
    else if (strcmp(filetype, "gatan") == 0) {
	return readGatan(filespec, nodata);
    }
    else if (strcmp(filetype, "gatan3") == 0) {
	return readGatan3(filespec, nodata);
    }
    else if (strcmp(filetype, "pif") == 0) {
	return readPIF(filespec, n, nodata);
    }
    else {
	printf("Error: invalid file type '%s' for file '%s'\n",
	       filetype, filespec);
	return -1;
    }
    return 0;
}

// return 0 if OK; 
int PyEMData::writeImage(const char* filespec, const char* filetype,
			 int n, int nolock, int mode)
{
    if (!filespec) {
	printf("Error: no file is input\n");
	return -1;
    }

    if (!filetype) {
	printf("Error: no filetype is specified\n");
	return -1;
    }

    if (strcmp(filetype, "img3d") == 0) {
	return writeIMAGIC3D(filespec, nolock);
    }
    else if (strcmp(filetype, "img") == 0) {
	return writeIMAGIC(filespec, n, nolock);
    }
    else if (strcmp(filetype, "icos") == 0) {
	return writeICOS(filespec);
    }
    else if (strcmp(filetype, "mrc") == 0) {
	return writeMRC(filespec, mode);
    }
    else if (strcmp(filetype, "spi") == 0) {
	return writeSPIDER(filespec, n);
    }
    else if (strcmp(filetype, "sspi") == 0) {
	return writeSingleSPIDER(filespec);
    }
    else if (strcmp(filetype, "emim") == 0) {
	return writeEMIM(filespec, n);
    }
    else if (strcmp(filetype, "vtk") == 0) {
	return writeVTK(filespec);
    }
    else if (strcmp(filetype, "amira") == 0) {
	return writeAMIRA(filespec);
    }
    else {
	printf("Error: invalid file type '%s' for file '%s'\n",
	       filetype, filespec);
	return -1;
    }
    return 0;
}


PyEMData_ptr PyEMData::py_copy(int withfft, int withparent)
{
    PyEMData* em = static_cast<PyEMData*> (copy(withfft, withparent));
    return PyEMData_ptr(em);
}


PyEMData_ptr PyEMData::py_copyHead()
{
    PyEMData* em = static_cast<PyEMData*> (copyHead());
    return PyEMData_ptr(em);
}


PyEMData_ptr PyEMData::py_clip(int x0, int y0, int w, int h)
{
    PyEMData* em = static_cast<PyEMData*> (clip(x0, y0, w, h));
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_clip(int x0, int y0, int z0, int w, int h, int d)
{
    PyEMData* em = static_cast<PyEMData*> (clip(x0, y0, z0, w, h, d));
    return PyEMData_ptr(em);
}

void PyEMData::py_insertClip(PyEMData_ptr block, int x0,  int y0)
{
    insertClip(block.get(), x0, y0);
}

    
void PyEMData::py_calcHist(PyList hist_list, int n, float hmin,
			   float hmax, int add)
{
#ifdef DEBUG
    assert(hist_list.size() >= n);
#endif
    
    float* hist_array = new float[hist_list.size()];
    list2array<float>(hist_list, hist_array);

    calcHist(hist_array, n, hmin, hmax, add);

    array2list<float>(hist_array, hist_list);
    delete [] hist_array;
    hist_array = 0;
}


void PyEMData::py_setParent(PyEMData_ptr data)
{
    setParent(data.get());
}

PyEMData_ptr PyEMData::py_Parent()
{
    PyEMData* em = static_cast<PyEMData*> (Parent());
    return PyEMData_ptr(em);
}

Euler_ptr PyEMData::py_getEuler()
{
    PyEuler* e = static_cast<PyEuler*> (getEuler());
    return Euler_ptr(e);
}

void PyEMData::py_setRAlign(Euler_ptr angle)
{
    setRAlign(angle.get());
}


imagicH_ptr PyEMData::py_getImagicHed()
{
    imagicH* ih = getImagicHed();
    return imagicH_ptr(ih);
}

void PyEMData::py_setImagicHed(imagicH_ptr hed)
{
    setImagicHed(hed.get());
}

mrcH_ptr PyEMData::py_getMRCHed()
{
    mrcH* mh = getMRCHed();
    return mrcH_ptr(mh);
}

SPIDERH_ptr PyEMData::py_getSpiHed()
{
    SPIDERH* sh = getSpiHed();
    return SPIDERH_ptr(sh);
}

  
PyEMData_ptr PyEMData::py_doFFT()
{
    PyEMData* em = static_cast<PyEMData*> (doFFT());
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_doIFT()
{
    PyEMData* em = static_cast<PyEMData*> (doIFT());
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_doRadon()
{
    PyEMData* em = static_cast<PyEMData*> (doRadon());
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_vertACF(int maxdy)
{
    PyEMData* em = static_cast<PyEMData*> (vertACF(maxdy));
    return PyEMData_ptr(em);
}


PyEMData_ptr PyEMData::py_convolute(PyEMData_ptr with)
{
    PyEMData* em = static_cast<PyEMData*> (convolute(with.get()));
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_calcCCF(PyEMData_ptr with, PyEMData_ptr filter)
{
    PyEMData* em = static_cast<PyEMData*> (calcCCF(with.get(), filter.get()));
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_calcCCFX(PyEMData_ptr with, int y0, int y1,
				   int nosum)
{
    PyEMData* em = static_cast<PyEMData*> (calcCCFX(with.get(), y0, y1, nosum));
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_calcMCF(PyEMData_ptr with, PyEMData_ptr filter)
{
    PyEMData* em = static_cast<PyEMData*> (calcMCF(with.get(), filter.get()));
    return PyEMData_ptr(em);
}


float PyEMData::py_transAlign(PyEMData_ptr with, int useparent, int intonly,
			      int maxshift, Float_ptr peak)
{
    float* p_peak = get_float_ptr(peak);
    return transAlign(with.get(), useparent, intonly, maxshift, p_peak);    
}

float PyEMData::py_rotAlign(PyEMData_ptr with)
{
    return rotAlign(with.get());
}

float PyEMData::py_rotAlignCH(PyEMData_ptr with, int irad, int orad)	
{
    return rotAlignCH(with.get(), irad, orad);
}

float PyEMData::py_transAlign3d(PyEMData_ptr with, int useparent, int intonly)	
{
    return transAlign3d(with.get(), useparent, intonly);
}


PyEMData_ptr PyEMData::py_RTFAlignRadon(PyEMData_ptr with, int maxshift,
					PyEMData_ptr thisf, PyEMData_ptr radonwith,
					PyEMData_ptr radonthis, PyEMData_ptr radonthisf)
{
    EMData* d = RTFAlignRadon(with.get(), maxshift, thisf.get(),
			      radonwith.get(), radonthis.get(), radonthisf.get());
    PyEMData* em = static_cast<PyEMData*> (d);
    return PyEMData_ptr(em);
}


PyEMData_ptr PyEMData::py_RTAlignRadon(PyEMData_ptr with, int maxshift,
				       PyEMData_ptr radonwith, PyEMData_ptr radonthis)
{
    EMData* d = RTAlignRadon(with.get(), maxshift, radonwith.get(), radonthis.get());
    PyEMData* em = static_cast<PyEMData*> (d);
    return PyEMData_ptr(em);
}


PyEMData_ptr PyEMData::py_RTFAlignSlow(PyEMData_ptr with, PyEMData_ptr flip,
				       int maxshift)
{
    EMData* d = RTFAlignSlow(with.get(), flip.get(), maxshift);
    PyEMData* em = static_cast<PyEMData*>(d); 
    return PyEMData_ptr(em);
}


PyEMData_ptr PyEMData::py_RTFAlignSlowest(PyEMData_ptr with, PyEMData_ptr flip,
					  int maxshift)
{
    EMData* d = RTFAlignSlowest(with.get(), flip.get(), maxshift);
    PyEMData* em = static_cast<PyEMData*>(d);
    return PyEMData_ptr(em);
}


PyEMData_ptr PyEMData::py_RTFAlign(PyEMData_ptr with, PyEMData_ptr flip,
				   int usedot, int maxshift)
{
    EMData* d = RTFAlign(with.get(), flip.get(), usedot, maxshift);
    PyEMData* em = static_cast<PyEMData*>(d);
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_RFAlign(PyEMData_ptr with, PyEMData_ptr flip, int imask)
{
    PyEMData* em = static_cast<PyEMData*> (RFAlign(with.get(), flip.get(), imask));
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_RTAlign(PyEMData_ptr with, int usedot, int maxshift)
{
    PyEMData* em = static_cast<PyEMData*> (RTAlign(with.get(), usedot,maxshift));
    return PyEMData_ptr(em);
}


void PyEMData::py_refineAlign(PyEMData_ptr with, int mode)
{
    refineAlign(with.get(), mode);
}

PyEMData_ptr PyEMData::py_makeRFP(int premasked, int unwrap)
{
    PyEMData* em = static_cast<PyEMData*> (makeRFP(premasked, unwrap));
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_unwrap(int r1, int r2, int xs,
				 int dx, int dy, int do360)   
{
    PyEMData* em = static_cast<PyEMData*> (unwrap(r1, r2, xs, dx, dy, do360));
    return PyEMData_ptr(em);
}

void PyEMData::py_calcRCF(PyEMData_ptr with, PyList sum_list, int NS)
{
#ifdef DEBUG
    assert(sum_list.size() >= NS);
#endif
    
    float* sum_array = new float[sum_list.size()];
    list2array<float>(sum_list, sum_array);    
    calcRCF(with.get(), sum_array, NS);
    array2list(sum_array, sum_list);
    delete [] sum_array;
    sum_array = 0;
}

void PyEMData::py_applyRadFn(int n, float x0, float dx, PyList y_list, int interp)
{
#ifdef DEBUG
    assert(y_list.size() >= n);
#endif
    
    float* y_array = new float[y_list.size()];
    list2array<float>(y_list, y_array);
    applyRadFn(n, x0, dx, y_array, interp);
    array2list<float>(y_array, y_list);
    delete [] y_array;
    y_array = 0;
}

void PyEMData::py_addRadNoise(int n, float x0, float dx, PyList y_list, int interp)	   
{
#ifdef DEBUG
    assert(y_list.size() >= n);
#endif
    
    float* y_array = new float[y_list.size()];
    list2array<float>(y_list, y_array);
    addRadNoise(n, x0, dx, y_array, interp);
    array2list<float>(y_array, y_list);
    delete [] y_array;
    y_array = 0;
}

void PyEMData::py_subtract(PyEMData_ptr data)	
{
    subtract(data.get());
}


void PyEMData::py_normalizeTo(PyEMData_ptr noisy, int keepzero,
			      int invert, Float_ptr mult, Float_ptr add)	
{
    float* p_mult = get_float_ptr(mult);
    float* p_add = get_float_ptr(add);
    normalizeTo(noisy.get(), keepzero, invert, p_mult, p_add);
}


void PyEMData::py_leastSquareNormalizeTo(PyEMData_ptr to,  float low_thresh,
					 float high_thresh, Float_ptr scale,
					 Float_ptr shift)
{
    float* p_scale = get_float_ptr(scale);
    float* p_shift = get_float_ptr(shift);
    leastSquareNormalizeTo(to.get(), low_thresh, high_thresh, p_scale, p_shift);
}

void PyEMData::py_maskNormalize(PyEMData_ptr mask, int sigmatoo) 
{
    maskNormalize(mask.get(), sigmatoo);
}

void PyEMData::py_calcRadDist(int n, float x0, float dx, PyList y_list)
{
#ifdef DEBUG
    assert(y_list.size() >= n);
#endif
    
    float* y_array = new float[y_list.size()];
    list2array(y_list, y_array);
    calcRadDist(n, x0, dx, y_array);
    array2list(y_array, y_list);
    delete [] y_array;
    y_array = 0;
}

void PyEMData::py_calcRadDist(int n, float x0, float dx, PyList y_list,
			      float acen, float amwid)
{
#ifdef DEBUG
    assert(y_list.size() >= 0);
#endif
    
    float* y_array = new float[y_list.size()];
    list2array<float>(y_list, y_array);
    calcRadDist(n, x0, dx, y_array, acen, amwid);
    array2list<float>(y_array, y_list);
    delete [] y_array;
    y_array = 0;
}

void PyEMData::py_calcAzDist(int n, float a0, float da, PyList d_list,
			     float rmin, float rmax)
{
#ifdef DEBUG
    assert(d_list.size() >= n);
#endif
    
    float* d_array = new float[d_list.size()];
    list2array(d_list, d_array);
    calcAzDist(n, a0, da, d_array, rmin, rmax);
    array2list(d_array, d_list);
    delete [] d_array;
    d_array = 0;
}

float PyEMData::py_dot(PyEMData_ptr data, int evenonly)
{
    return dot(data.get(), evenonly);
}

float PyEMData::py_lcmp(PyEMData_ptr noisy, int keepzero, int invert, 
			Float_ptr scale, Float_ptr shift)
{
    float* p_scale = get_float_ptr(scale);
    float* p_shift = get_float_ptr(shift);    
    return lcmp(noisy.get(), keepzero, invert, p_scale, p_shift);
}

float PyEMData::py_pcmp(PyEMData_ptr data, XYData_ptr snr)
{
    return pcmp(data.get(), snr.get());
}


PyList PyEMData::py_ctfCurve(int type, XYData_ptr sf)
{
    float* sf_array = ctfCurve(type, sf.get());
    // convert sf-array to a list????
    return PyList();
}

float PyEMData::py_fscmp(PyEMData_ptr with)
{
    return fscmp(with.get());
}

PyList PyEMData::py_fsc(PyEMData_ptr with)
{
    // return NULL or array: size = ?
    float* array = fsc(with.get());
    return PyList();
    
}



void PyEMData::py_makeAverageWT(PyList in_pylist, PyEMData_ptr curves, XYData_ptr sf)
{
    List* in_emlist = new List();
    pylist2emlist(in_pylist, in_emlist);

    makeAverageWT(in_emlist, curves.get(), sf.get());

    delete in_emlist;
    in_emlist = 0;
}

void PyEMData::py_makeAverageCTFCW(PyList in_pylist, XYData_ptr sf, PyList SNR_list)
{
    List* in_emlist = new List();
    pylist2emlist(in_pylist, in_emlist);
    
    int sz = SNR_list.size();
    float* SNR_array = new float[sz];
    list2array(SNR_list, SNR_array);

    makeAverageCTFCW(in_emlist, sf.get(), SNR_array);

    array2list(SNR_array, SNR_list);
    delete [] SNR_array;
    SNR_array = 0;

    delete in_emlist;
    in_emlist = 0;
}

void PyEMData::py_makeAverage(PyList in_pylist, PyEMData_ptr sigma)
{
    List* in_emlist = new List();
    pylist2emlist(in_pylist, in_emlist);

    makeAverage(in_emlist, sigma.get());

    delete in_emlist;
    in_emlist = 0;
}

void PyEMData::py_makeAverageIter(PyList in_pylist)
{
    List* in_emlist = new List();
    pylist2emlist(in_pylist, in_emlist);
    makeAverageIter(in_emlist);

    delete in_emlist;
    in_emlist = 0;
}


void PyEMData::py_makeAverageCTFC(PyList in_pylist, float filter)
{
    List* in_emlist = new List();
    pylist2emlist(in_pylist, in_emlist);

    makeAverageCTFC(in_emlist, filter);

    delete in_emlist;
    in_emlist = 0;
}

void PyEMData::py_makeAverageCTFCWauto(PyList in_pylist, const char *outfile)
{
    List* in_emlist = new List();
    pylist2emlist(in_pylist, in_emlist);

    makeAverageCTFCWauto(in_emlist, const_cast<char*>(outfile));
    
    delete in_emlist;
    in_emlist = 0;
}


void PyEMData::py_makeMedian(PyList in_pylist)
{
    List* in_emlist = new List();
    pylist2emlist(in_pylist, in_emlist);

    makeMedian(in_emlist);

    delete in_emlist;
    in_emlist = 0;
}

void PyEMData::py_add(PyEMData_ptr in)
{
    add(in.get());
}

void PyEMData::py_addIncoherent(PyEMData_ptr in)						
{
    addIncoherent(in.get());
}

void PyEMData::py_mult(PyEMData_ptr in, int recip)
{
    mult(in.get(), recip);
}

void PyEMData::py_commonLines(PyEMData_ptr data1, PyEMData_ptr data2, int mode, 
			      int steps, int horiz)
{
    commonLines(data1.get(), data2.get(), mode, steps, horiz);
}

void PyEMData::py_commonLinesR(PyEMData_ptr data1, PyEMData_ptr data2, 
			       int steps, int horiz)				
{
    commonLinesR(data1.get(), data2.get(), steps, horiz);
}

PyEMData_ptr PyEMData::py_project3d(float alt, float az,  float phi, 
				    int mode,  float thr)
{
    PyEMData* em = static_cast<PyEMData*> (project3d(alt, az, phi,mode, thr));
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_fftSlice(float alt, float az, float phi, int mode)
{
    PyEMData* em = static_cast<PyEMData*> (fftSlice(alt, az, phi, mode));
    return PyEMData_ptr(em);
}

void PyEMData::py_interpFT3d(float x, float y, float z, PyList ret_list, int mode)
{
    // only 2 elements are used in ret_list
#ifdef DEBUG
    assert(ret_list.size() >= 2);
#endif
    float* ret_array =  new float[ret_list.size()];
    list2array(ret_list, ret_array);
    interpFT3d(x, y, z, ret_array, mode);
    array2list(ret_array, ret_list);
    delete [] ret_array;
    ret_array = 0;
}


float PyEMData::py_normSlice(PyEMData_ptr slice, float alt, float az,
			     float phi, Float_ptr phaseresid)
{
    float* p_phaseresid = get_float_ptr(phaseresid);
    return normSlice(slice.get(), alt, az, phi, p_phaseresid);
}


void PyEMData::py_insertSlice(PyEMData_ptr slice, float alt, float az,
			      float phi, int mode, float weight)
{
    insertSlice(slice.get(), alt, az, phi, mode, weight);
}


void PyEMData::py_insertSliceWF(PyEMData_ptr slice, float alt, float az,
				float phi, int mode, PyList SNR_list,
				float padratio)
{
    float* SNR_array = new float[SNR_list.size()];
    list2array(SNR_list, SNR_array);
    insertSliceWF(slice.get(), alt, az, phi, mode, SNR_array, padratio);
    array2list(SNR_array, SNR_list);
    delete [] SNR_array;
    SNR_array = 0;
}

void PyEMData::py_insertSlice3DReal(PyEMData_ptr slice, float weight)

{
    insertSlice3DReal(slice.get(), weight);
}


PyList PyEMData::py_getCTF()
{
    // length = 11
    const int SZ = 11;
    float* ctf_array = getCTF();
    PyList ctf_list(SZ);
    array2list(ctf_array, ctf_list);
    return ctf_list;
}

void PyEMData::py_setCTF(PyList ctf_list)
{
    // length = 11
    const int SZ = 11;
#ifdef DEBUG
    assert(ctf_list.size() == SZ);
#endif
    float* ctf_array = new float[SZ];
    list2array<float>(ctf_list, ctf_array);
    setCTF(ctf_array);
    delete [] ctf_array;
    ctf_array = 0;
}

int PyEMData::py_CTFCmp(PyEMData_ptr with)
{
    return CTFCmp(with.get());
}

PyEMData_ptr PyEMData::py_iftSlice()
{
    PyEMData* em = static_cast<PyEMData*> (iftSlice());
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_row(int n)
{
    PyEMData* em = static_cast<PyEMData*> (row(n));
    return PyEMData_ptr(em);
}

PyEMData_ptr PyEMData::py_col(int n)
{
    PyEMData* em = static_cast<PyEMData*> (col(n));
    return PyEMData_ptr(em);
}

void PyEMData::py_setRow(PyEMData_ptr d, int n)
{
    setRow(d.get(), n);
}

void PyEMData::py_setCol(PyEMData_ptr d, int n)
{
    setCol(d.get(), n);
}

PyEMData_ptr PyEMData::py_mxMult(PyEMData_ptr second)
{
    PyEMData* em = static_cast<PyEMData*> (mxMult(second.get()));
    return PyEMData_ptr(em);
}


float PyEMData::py_dist(PyEMData_ptr second, int n)
{
    return dist(second.get(), n);
}
