#ifndef _pyemdata_h_
#define _pyemdata_h_

#include "EMData.h"
#include "PyList.h"
#include "PyEuler.h"
#include <boost/smart_ptr.hpp>


class PyEMData;

typedef boost::shared_ptr<PyEMData> PyEMData_ptr;
typedef boost::shared_ptr<imagicH> imagicH_ptr;
typedef boost::shared_ptr<mrcH> mrcH_ptr;
typedef boost::shared_ptr<SPIDERH> SPIDERH_ptr;
typedef boost::shared_ptr<XYData> XYData_ptr;
typedef boost::shared_ptr<PyEuler> Euler_ptr;


class PyEMData : public EMData {
public:
    PyEMData();
    ~PyEMData();
    
    int readImage(const char* filespec, const char* filetype = 0,
		  int n = 0, int nodata = 0);
    
    int writeImage(const char* filespec, const char* filetype,
		   int n = 0, int nolock = 0, int mode = MODE_float);
    
    PyEMData_ptr py_copy(int withfft = 0, int withparent =0);
    PyEMData_ptr py_copyHead();
    
    PyEMData_ptr py_clip(int x0, int y0, int w, int h);
    PyEMData_ptr py_clip(int x0, int y0, int z0, int w, int h, int d);
    void py_insertClip(PyEMData_ptr block, int x0,  int y0);
    
    void py_calcHist(PyList hist_list, int n, float hmin = 0,
		     float hmax = 0, int add = 0);

    void py_setParent(PyEMData_ptr data);
    PyEMData_ptr py_Parent();

    
    Euler_ptr py_getEuler(); 
    void py_setRAlign(Euler_ptr angle);

    imagicH_ptr py_getImagicHed();
    void py_setImagicHed(imagicH_ptr hed);
    mrcH_ptr py_getMRCHed();
    SPIDERH_ptr py_getSpiHed();
    
    PyEMData_ptr py_doFFT();
    PyEMData_ptr py_doIFT();
    PyEMData_ptr py_doRadon();
    PyEMData_ptr py_vertACF(int maxdy);

    PyEMData_ptr py_convolute(PyEMData_ptr with);
    PyEMData_ptr py_calcCCF(PyEMData_ptr with, PyEMData_ptr filter = PyEMData_ptr(0));
    PyEMData_ptr py_calcCCFX(PyEMData_ptr with, int y0 = 0, int y1 = -1, int nosum = 0);
    PyEMData_ptr py_calcMCF(PyEMData_ptr with, PyEMData_ptr filter = PyEMData_ptr(0));

    float py_transAlign(PyEMData_ptr with, int useparent, int intonly,
			int maxshift, Float_ptr peak = Float_ptr(0));
    float py_rotAlign(PyEMData_ptr with);	
    float py_rotAlignCH(PyEMData_ptr with, int irad = 8, int orad = 0);	
    float py_transAlign3d(PyEMData_ptr with, int useparent = 0, int intonly = 0);	

    PyEMData_ptr py_RTFAlignRadon(PyEMData_ptr with, int maxshift = -1,
				  PyEMData_ptr thisf = PyEMData_ptr(0),
				  PyEMData_ptr radonwith = PyEMData_ptr(0),
				  PyEMData_ptr radonthis = PyEMData_ptr(0),
				  PyEMData_ptr radonthisf = PyEMData_ptr(0));

    PyEMData_ptr py_RTAlignRadon(PyEMData_ptr with, int maxshift = -1,
				 PyEMData_ptr radonwith = PyEMData_ptr(0),
				 PyEMData_ptr radonthis = PyEMData_ptr(0));

    PyEMData_ptr py_RTFAlignSlow(PyEMData_ptr with,
				 PyEMData_ptr flip = PyEMData_ptr(0),
				 int maxshift = -1);

    PyEMData_ptr py_RTFAlignSlowest(PyEMData_ptr with,
				    PyEMData_ptr flip = PyEMData_ptr(0),
				    int maxshift = -1);

    PyEMData_ptr py_RTFAlign(PyEMData_ptr with = PyEMData_ptr(0),
			     PyEMData_ptr flip = PyEMData_ptr(0),
			     int usedot = 1, int maxshift = -1);

    PyEMData_ptr py_RFAlign(PyEMData_ptr with,
			    PyEMData_ptr flip = PyEMData_ptr(0),
			    int imask = 0);
    
    PyEMData_ptr py_RTAlign(PyEMData_ptr with, int usedot = 0, int maxshift = 0);

    void py_refineAlign(PyEMData_ptr with, int mode = 1);
    PyEMData_ptr py_makeRFP(int premasked = 0, int unwrap = 1);
    PyEMData_ptr py_unwrap(int r1 = -1, int r2 = -1, int xs = -1,
			   int dx = 0, int dy = 0, int do360 = 0);
    
    void py_calcRCF(PyEMData_ptr with, PyList sum_list, int NS);
    void py_applyRadFn(int n, float x0, float dx, PyList y_list, int interp = 1);
    void py_addRadNoise(int n, float x0, float dx, PyList y_list, int interp);	   
    void py_subtract(PyEMData_ptr data);
    
    void py_normalizeTo(PyEMData_ptr noisy, int keepzero = 0, int invert = 0,
			Float_ptr mult = Float_ptr(0),
			Float_ptr add = Float_ptr(0));	

    void py_leastSquareNormalizeTo(PyEMData_ptr to, float low_thresh = FLT_MIN, 
				   float high_thresh = FLT_MAX,
				   Float_ptr scale = Float_ptr(0),
				   Float_ptr shift = Float_ptr(0));

    void py_maskNormalize(PyEMData_ptr mask, int sigmatoo = 1); 
    void py_calcRadDist(int n, float x0, float dx, PyList y_list);
    void py_calcRadDist(int n, float x0, float dx, PyList y_list,
			float acen, float amwid);

    void py_calcAzDist(int n, float a0, float da, PyList data_list,
		       float rmin, float rmax);
    float py_dot(PyEMData_ptr data, int evenonly = 0);
    float py_lcmp(PyEMData_ptr noisy, int keepzero = 0, int invert = 0,
		  Float_ptr scale = Float_ptr(0),
		  Float_ptr shift = Float_ptr(0));	 

    float py_pcmp(PyEMData_ptr data, XYData_ptr snr = XYData_ptr(0));
    void py_makeAverageWT(PyList in, PyEMData_ptr curves, XYData_ptr sf);
    void py_makeAverageCTFCW(PyList in, XYData_ptr sf, PyList SNR_list = PyList(0));    
    PyList py_ctfCurve(int type, XYData_ptr sf = XYData_ptr(0));
    
    float py_fscmp(PyEMData_ptr with);
    PyList py_fsc(PyEMData_ptr with);
    void py_makeAverage(PyList in, PyEMData_ptr sigma = PyEMData_ptr(0));
    void py_makeAverageIter(PyList in);
    
    void py_makeAverageCTFC(PyList in, float filtr);
    void py_makeAverageCTFCWauto(PyList in, const char *outfile = 0);
    
    void py_makeMedian(PyList in);
    void py_add(PyEMData_ptr in);
    void py_addIncoherent(PyEMData_ptr in);
						
    void py_mult(PyEMData_ptr in, int recip = 0);

    void py_commonLines(PyEMData_ptr d1, PyEMData_ptr d2, int mode = 0, 
			int steps = 180, int horiz = 0);
    
    void py_commonLinesR(PyEMData_ptr data1, PyEMData_ptr data2, 
			 int steps = 180, int horiz = 0);
				
    PyEMData_ptr py_project3d(float alt, float az,  float phi, 
			      int mode,  float thr = 1.0);
    PyEMData_ptr py_fftSlice(float alt, float az, float phi, int mode = 5);
    void py_interpFT3d(float x, float y, float z, PyList ret_list, int mode);

    float py_normSlice(PyEMData_ptr slice, float alt, float az,
		       float phi, Float_ptr phaseresid = Float_ptr(0));

    void py_insertSlice(PyEMData_ptr slice, float alt, float az,
			float phi, int mode = 5, float weight = 1.0);

    void py_insertSliceWF(PyEMData_ptr slice, float alt, float az,
			  float phi, int mode, PyList SNR_list,
			  float padratio);
    void py_insertSlice3DReal(PyEMData_ptr slice, float weight);

    PyList py_getCTF();
    void py_setCTF(PyList ctf);
    
    int py_CTFCmp(PyEMData_ptr with);
    PyEMData_ptr py_iftSlice();
    PyEMData_ptr py_row(int n);
    PyEMData_ptr py_col(int n);
    void py_setRow(PyEMData_ptr d, int n);
    void py_setCol(PyEMData_ptr d, int n);
    PyEMData_ptr py_mxMult(PyEMData_ptr second);
    float py_dist(PyEMData_ptr second, int n = 0);

};

int fileCount(const char* filespec, String& filetypes);

void pylist2emlist(const PyList& pylist, List* emlist);
void emlist2pylist(List* emlist, PyList& pylist);


#endif
