#include "PyEuler.h"

#if DEBUG
#include "assert.h"
#endif

PyEuler::PyEuler()
    : Euler()
{

}

PyEuler::PyEuler(float a1, float a2, float a3)
    : Euler(a1, a2, a3)
{
}

PyEuler::PyEuler(float a1, float a2, float a3, int typ) :
    Euler(a1, a2, a3, typ)
{
}

PyEuler::PyEuler(float a1, float a2, float a3, float a0, int typ) :
    Euler(a1, a2, a3, a0, typ)
{
}

PyEuler::PyEuler(PyList m_list)
{
    py_setAngle(m_list);
    init();
}

PyEuler::~PyEuler()
{
}


void PyEuler::py_setAngle(PyList m_list)
{
#if DEBUG
    assert(m_list.size() == EULER_MATRIX_SIZE);
#endif

    float* m_array = new float[EULER_MATRIX_SIZE];
    list2array<float>(m_list, m_array);
    setAngle(m_array);
    array2list<float>(m_array, m_list);
    delete [] m_array;
    m_array = 0;
}

void PyEuler::py_setSym(const char *symmetry)
{
    setSym(const_cast<char*>(symmetry));
}

PyList PyEuler::py_mat()
{
    float* m_array = mat();
    PyList m_list(EULER_MATRIX_SIZE);
    
    array2list<float>(m_array, m_list);
    return m_list;
}

void PyEuler::py_print(const char *id, FILE* out)
{
    print(const_cast<char*>(id), out);
}

void PyEuler::py_printMatrix(const char *id, FILE* out)
{
    printMatrix(const_cast<char*>(id), out);
}



PyEuler operator*(PyEuler lhs, PyEuler rhs)
{
    Euler e1 = lhs;
    Euler e2 = rhs;
    Euler r =  e1 * e2;
    return *(static_cast<PyEuler*>(&r));
}

PyEuler operator/(PyEuler lhs, PyEuler rhs)
{
    Euler e1 = lhs;
    Euler e2 = rhs;
    Euler r = e1 / e2;
    return *(static_cast<PyEuler*>(&r));
}

bool operator!=(PyEuler lhs, PyEuler rhs)
{
    Euler e1 = lhs;
    Euler e2 = rhs;
    return (e1 != e2);
}


bool operator==(PyEuler lhs, PyEuler rhs)
{
    return !(lhs != rhs);
}

