#include "PyVect.h"

PyQuater::PyQuater() : Quater() {}

PyQuater::PyQuater(PyEuler& e) : Quater(e) { }

PyQuater::PyQuater(float x, float y, float z, float q) :
    Quater(x, y, z, q) { }

void PyQuater::py_fromEuler(PyEuler& e)
{
    fromEuler(e);
}

PyEuler PyQuater::py_toEuler()
{
    Euler e = toEuler();
    PyEuler* pye = static_cast<PyEuler*>(&e);
    return *pye;
}

void PyQuater::py_print(const char* id, FILE* out)
{
    print(const_cast<char*>(id), out);
}

// class Vect

PyVect::PyVect() : Vect()
{
}

PyVect::PyVect(float xx, float yy, float zz) : Vect(xx, yy, zz)
{
    
}

PyVect PyVect::py_rotate(PyList m_list)
{
    float *m_array = new float[PyEuler::EULER_MATRIX_SIZE];
    list2array<float>(m_list, m_array);
    Vect v = rotate(m_array);
    array2list<float>(m_array, m_list);
    delete [] m_array;
    m_array = 0;
    
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(PyEuler& ang)
{
    Vect v = rotate(ang);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(PyQuater& q)
{
    Vect v = rotate(q);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(PyVect& p, float q)
{
    Vect v = rotate(p, q);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(float alt, float az, float phi)
{
    Vect v = rotate(alt, az, phi);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_rotate(float nx, float ny, float nz, float q)
{
    Vect v = rotate(nx, ny, nz, q);
    return *(static_cast<PyVect*>(&v));
}

PyVect PyVect::py_cross(PyVect& p)
{
    Vect v = cross(p);
    return *(static_cast<PyVect*>(&v));
}

PyQuater PyVect::py_getQuater()
{
    Quater q = getQuater();
    return *(static_cast<PyQuater*>(&q));
}

float PyVect::py_inner(PyVect& p)
{
    return inner(p);
}

float PyVect::py_angle(PyVect& p, PyVect y)
{
    return angle(p, y);
}

PyEuler PyVect::py_euler()
{
    Euler e = euler();
    return *(static_cast<PyEuler*>(&e));
}

PyEuler PyVect::py_eulerTo(PyVect& p)
{
    Euler e = eulerTo(p);
    return *(static_cast<PyEuler*>(&e));
}

PyEuler PyVect::py_eulerTo(float x, float y, float z)
{
    Euler e = eulerTo(x, y, z);
    return *(static_cast<PyEuler*>(&e));
}

bool PyVect::py_isNeighbor(PyVect& p)
{
    isNeighbor(p);
}

void PyVect::py_print(const char* id, FILE* out)
{
    print(const_cast<char*>(id), out);
}


PyVect operator * (const PyVect& lhs, float f)
{
    Vect v = lhs;
    Vect r = v * f;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator / (const PyVect& lhs, float f)
{
    Vect v = lhs;
    Vect r = v / f;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator + (const PyVect& lhs, float f)
{
    Vect v = lhs;
    Vect r = v + f;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator - (const PyVect& lhs, float f)
{
    Vect v = lhs;
    Vect r = v - f;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator + (const PyVect& lhs, const PyVect& rhs)
{
    Vect v1 = lhs;
    Vect v2 = rhs;
    Vect r = v1 + v2;
    return *(static_cast<PyVect*>(&r));
}

PyVect operator - (const PyVect& lhs, const PyVect& rhs)
{
    Vect v1 = lhs;
    Vect v2 = rhs;
    Vect r = v1 - v2;
    return *(static_cast<PyVect*>(&r));
}

bool operator == (const PyVect& lhs, const PyVect& rhs)
{
    Vect v1 = lhs;
    Vect v2 = rhs;
    return (v1 == v2);
}

