#include <stdio.h>
#include "Vect.h"

// by Wen Jiang, June 2000

/**************************************************************************
			Vect class	
***************************************************************************/

Vect::Vect() 
{
	x = y = z = 0;
}

Vect::Vect(float xx, float yy, float zz) 
{
	x = xx;
	y = yy;
	z = zz;
}

void Vect::setX(float xx)
{
	x=xx;
}

void Vect::setY(float yy)
{
	y=yy;
}

void Vect::setZ(float zz)
{
	z=zz;
}

void Vect::setXYZ(float xx, float yy, float zz)
{
	x=xx; y=yy; z=zz;
}

bool Vect::isNeighbor(Vect * p) 
{	//26 neighbor connection
	if (fabs(x - p->X()) <= 1 && fabs(y - p->Y()) <= 1 && fabs(z - p->Z()) <= 1)
		return true; 
	else 
		return false;
}	

bool Vect::isNeighbor(Vect & p) 
{	//26 neighbor connection
	if (fabs(x - p.X()) <= 1 && fabs(y - p.Y()) <= 1 && fabs(z - p.Z()) <= 1)
		return true; 
	else 
		return false;
}	

void Vect::normalize() 
{
	float	norm = sqrt(x*x + y*y + z*z);
	x /= norm;
	y /= norm;
	z /= norm;
}

float Vect::length() 
{
	return(sqrt(x*x + y*y + z*z));
}

Vect Vect::rotate(float nx, float ny, float nz, float q) 
{
	Quater t=Quater(nx, ny, nz, q);
	return(rotate(t));
}

// this should be right according to Euler matrix mult
Vect Vect::rotate(float m[9])	
{
	float	tx, ty, tz;
	tx = x * m[0] + y * m[1] + z * m[2];
	ty = x * m[3] + y * m[4] + z * m[5];
	tz = x * m[6] + y * m[7] + z * m[8];
	return(Vect(tx, ty, tz));
}

Vect Vect::rotate(float m[3][3])
{
	float	tx, ty, tz;
	tx = x * m[0][0] + y * m[0][1] + z * m[0][2];
	ty = x * m[1][0] + y * m[1][1] + z * m[1][2];
	tz = x * m[2][0] + y * m[2][1] + z * m[2][2];
	return(Vect(tx, ty, tz));
}

Vect Vect::rotate(Euler & ang)
{
	float	*m = ang.mat();
	return(rotate(m));
}

Vect Vect::rotate(Euler *ang)
{
	float	*m = ang->mat();
	return(rotate(m));
}


Vect Vect::rotate(float alt, float az, float phi)
{
	return(rotate(new Euler(alt, az, phi)));
}

Vect Vect::rotate(Quater *q)
{
	Euler t=q->toEuler();
	return(rotate(t));
}

Vect Vect::rotate(Quater& q)
{
	Euler t=q.toEuler();
	return(rotate(t));
}

Vect Vect::rotate(Vect& p, float q)
{
	Quater t=Quater(p.X(), p.Y(), p.Z(), q);
	return(rotate(t));
}

Quater Vect::getQuater()
{	
	Euler tmp=euler();
	return Quater(tmp);
}

Euler Vect::euler()
{
	if(x==y && x==z && x==0) return(Euler(0,0,0));
	
	float	az, cosalt, alt , phi;
	//az = atan2(y, x)+PI/2.;		//original, correct but nonstational  
	az = atan2(y,x)-PI/2;	//current, correct and stational
	cosalt = z / sqrt(x * x + y * y + z * z);
	if (cosalt > 1)
		cosalt = 1; 
	else if (cosalt < -1)
		cosalt = -1;
	
	if(cosalt>0.999999)return(Euler(0,0,0));
	
	alt = acos(cosalt);

	//return(Euler(alt,az,0));	//original, correct but nonstational  
	return(Euler(alt,az,-az));		//current, correct and stational
}	

Euler Vect::eulerTo(float x, float y, float z)
{
	Vect t=Vect(x, y, z);
	return(eulerTo(t));
}

Euler Vect::eulerTo(Vect& p)
{
	return p.euler()*euler().inverse();
}

Vect Vect::cross(Vect& p)
{ // =this x P
	Vect ret;
	ret.x = y * p.Z() - z * p.Y();
	ret.y = -x * p.Z() + z * p.X();
	ret.z = x * p.Y() - y * p.X();
	if (asin(ret.length() / (length() * p.length())) * 180 / PI < 1E-8) {
		ret = Vect(0, 0, 0);
		fprintf(stderr, "Vect::cross(Vect p): warning: the two vectors (%f,%f,%f) and (%f,%f,%f) are parrallel !\n",
		     x, y, z, p.X(), p.Y(), p.Z());
	}
	return(ret);
}


float	Vect::inner(Vect& p)
{ // =this . P
	return(x * p.X() + y * p.Y() + z * p.Z());
}


float	Vect::angle(Vect& p, Vect y0)
{ // think this as +x axis, y0 as the +y axis, angle (counterclockwise) from this to p in the plane
	float	denom = length()*p.length();
	if ((length() < 1E-8) ^ (p.length() < 1E-8)) {
		fprintf(stderr, "Vect::angle(Vect p, Vect y0): warning: one of the two vectors (%f,%f,%f) and (%f,%f,%f) is ZERO vector !\n",
		     x, y, z, p.X(), p.Y(), p.Z());
		return 0;	// set to 0 for now to signal bad vectors
	} else {
		float	cosang = inner(p) / denom;	
		if (cosang > 1)
			cosang = 1;
		else if (cosang < -1)
			cosang = -1;
		float	ret = acos(cosang);	// 0-Pi
		if (y0.length() > 0) {
			if (y0.inner(p) < 0) 
				ret = 2 * PI - ret;	// 0-2Pi
		}
		return(ret);
	}
}

void Vect::print(char* id, FILE* out)
{
	fprintf(out,"Vect %s =( %.4f , %.4f , %.4f )\n",id,x,y,z);
}

Vect Vect::operator * (float r) 
{
	return Vect(x * r, y * r, z * r);
}

Vect Vect::operator / (float r) 
{
	return Vect(x / r, y / r, z / r);
}

Vect Vect::operator + (float r) 
{
	return Vect(x + r, y + r, z + r);
}

Vect Vect::operator + (const Vect& r) 
{
	return Vect(x + r.x, y + r.y, z + r.z);
}

Vect Vect::operator - (float r) 
{
	return Vect(x - r, y - r, z - r);
}

Vect Vect::operator - (const Vect& r) 
{
	return Vect(x - r.x, y - r.y, z - r.z);
}

bool Vect::operator == (const Vect& p) 
{
	return (x == p.x) && (y == p.y) && (z = p.z);
}

/**************************************************************************
			Quaternion class	
***************************************************************************/

Quater::Quater()
{
	nx = 0;	
	ny = 0;	
	nz = 1;	
	Q = 0;
}

Quater::Quater(float x, float y, float z, float q)
{
	float	norm = sqrt(x*x + y*y + z*z);
	if(1.==norm+1.) {
		fprintf(stderr,"ERROR:: Quater::Quater(float x, float y, float z, float q): x,y,z cannot all be ZERO !\n");
		exit(1);
	}
	nx = x / norm;	
	ny = y / norm;	
	nz = z / norm;	
	Q = q;
	//printf("x,y,z=%f,%f,%f\tnorm=%f\tquater=%f,%f,%f,%f\n",x,y,z,norm,nx,ny,nz,Q);
}

Quater::Quater(Euler& e) {
	fromEuler(e);
} 

void	Quater::fromEuler(Euler& e)
{
	float	alt0 = e.alt();	//-alt;
	float	az0 = e.az();	//-phi;
	float	phi0 = e.phi();	//-az;
	// zxz convention
	float	e0 = cos((az0 + phi0) / 2.)*cos(alt0 / 2.);
	float	e1 = cos((az0-phi0) / 2.)*sin(alt0 / 2.);
	float	e2 = sin((az0-phi0) / 2.)*sin(alt0 / 2.);
	float	e3 = sin((az0 + phi0) / 2.)*cos(alt0 / 2.);

	if (e0 > 1.)
		e0 = 1.;
	if (e0 < -1.)
		e0 = -1.;

	float	norm = sqrt(1-e0*e0);
	if (1 + norm != 1) {
		Q	 = 	acos(e0) * 2.0;
		nx	 = 	e1 / norm;
		ny	 = 	e2 / norm;
		nz	 = 	e3 / norm;
	} else {
		Q	 = 	0;
		nx	 = 	0;
		ny	 = 	0;
		nz	 = 	0;
	}
}


Euler Quater::toEuler()
{
	float	alt, az;
	float	t = nz / (sqrt(nx*nx + ny*ny + nz*nz));
	if (t > 1)
		t = 1;	
	else if (t < -1)
		t = -1;
	alt = acos(t);
	az = atan2(ny, nx);	// -Pi -> Pi, correct full @PI range
	Euler r = Euler(alt, az + PI / 2, -PI / 2);
	Vect res = Vect(nx, ny, nz).rotate(r);
	//printf("nx,ny,nz=%f,%f,%f\talt=%f\taz=%f\t",nx,ny,nz,alt*180/PI,az*180/PI);
	//printf("Quater::toEuler():r=\t(alt,az,phi)=(\t%f\t%f\t%f\t)",r.alt()*180/PI,r.az()*180/PI,r.phi()*180/PI);		
	//printf("\tnew vector=(\t%f\t%f\t%f\t)\n",res.x,res.y,res.z);
	return(r.inverse() * Euler(0, Q, 0) * r);
}

void Quater::print(char* id, FILE* out)
{
	fprintf(out,"Quaternion %s = %f\t%f\t%f\t%f\n",id,nx,ny,nz,Q);
}

Quater euler2quater(Euler& ang)
{
	return(Quater(ang));
}


Quater euler2quater(float alt, float az, float phi)
{
	Euler t=Euler(alt, az, phi);
	return(euler2quater(t));
}


Euler quater2euler(float nx, float ny, float nz, float Q)
{
	return(Quater(nx, ny, nz, Q).toEuler());
}


Euler quater2euler(Quater& quater)
{
	return(quater.toEuler());
}


Euler quater2euler(Vect& p, float q)
{
	return(Quater(p.X(),p.Y(),p.Z(),q).toEuler());
}

/*********************************************
**
**		Transform3D class
**
*********************************************/
/*
Transform3D::Transform3D()
{
	setUnit();
}

Transform3D::Transform3D(Euler& euler)
{
	setUnit();
	setRotation(euler);
}

Transform3D::Transform3D(Vect& vect)
{
	setUnit();
	setTranslation(vect);	
}

Transform3D::Transform3D(Euler& euler, Vect& vect)
{
	setUnit();
	setRotation(euler);
	setTranslation(vect);	
}

Transform3D::Transform3D(const float mat[16])
{
	setMatrix(mat);
}

void Transform3D::setUnit()
{
	int i;
	for(i=0;i<16;i++)m[i]=0;
	for(i=0;i<4;i++)m[i*4+i]=1;
}

void Transform3D::setMatrix(const float mat[16])
{
 	for (int i = 0; i < 16; i++)
		m[i] = mat[i];
}

void Transform3D::setRotation(Euler& euler)
{
	const float *em=euler.mat();
	for(int i=0;i<3;i++)
		for(int j=0;j<3;j++)
			m[j*4+i]=em[j*3+i];
}

void Transform3D::setTranslation(Vect& vect)
{
	m[0*4+3]=vect.X();
	m[1*4+3]=vect.Y();
	m[2*4+3]=vect.Z();
}

Euler Transform3D::getRotation()
{
	float em[9];
	for(int i=0;i<9;i++) em[i]=m[i/3*4+i%3];
	return Euler(em);
}

Vect Transform3D::getTranslation()
{
	return Vect(m[0*4+3],m[1*4+3],m[2*4+3]);
}
		
const float *Transform3D::mat()
{
	return m;
}

Transform3D Transform3D::inverse()
{
	fprintf(stderr,"Transform3D Inverse(): not implemented yet\n");
	return NULL;
}

Transform3D Transform3D::inverseRotation()
{
	float rm[16];
	for(int i=0;i<16;i++)rm[i]=m[i];
	for(i=0;i<3;i++)
		for(int j=0;j<3;j++)
			rm[j*4+i]=m[i*4+j];
	return Transform3D(rm);	
}

void Transform3D::print(char* id, FILE* out)
{
	fprintf(out,"Transform3D matrix: %s\n",id);
	for (int j = 0; j < 4; j++){      // print by columns first
		for (int i = 0; i < 4; i++)  // then by rows
			fprintf(out,"%f\t",m[i*4+j]);
		fprintf(out,"\n");
	}
	fprintf(out,"\n");
}

Transform3D Transform3D::operator*(Transform3D& tf)
{
	float temp[16]; // temporary matrix for storing result
    const float *b=tf.mat();              
	int i, j;         // row and column counters
		for (i = 0; i < 4; i++)      // transform by rows first
			for (j = 0; j < 4; j++)  // then by columns
				temp[j*4+i]= b[0*4+i] * m[j*4+0] + b[1*4+i] * m[j*4+1] +
                             b[2*4+i] * m[j*4+2] + b[3*4+i] * m[j*4+3];
				//temp[j*4+i]= m[0*4+i] * b[j*4+0] + m[1*4+i] * b[j*4+1] +
                //             m[2*4+i] * b[j*4+2] + m[3*4+i] * b[j*4+3];
	return Transform3D(temp);
}

void Transform3D::operator=(Transform3D& tf)
{
    const float *b=tf.mat();              
	for (int i = 0; i < 16; i++) // copy result matrix into matrix m
		m[i]=b[i];
}
*/
