// VolFrag.C   Copyright 2000   Steve Ludtke		7/20/00
// This object contains a 3d pie slice of a spherical volume
// This can be used to store objects with high symmetry more
// efficiently in memory
// This object can only contain a limited subset of possible volumes
// all volumes contained in the set begin at x=0 and go to an arbitrary
// xmax. Xmax varies as a function of y and z.

#include <string.h>
#include "VolFrag.h"

VolFrag::VolFrag() {
size=0;
xmax=NULL;
data=NULL;
}

VolFrag::~VolFrag() {
free(xmax);
for (int i=0; i<size; i++) free(data[i]);
free(data);
}

void VolFrag::setSize(int s) {
int i;
for (i=0; i<size*size; i++) free(data[i]);

size=s;
xmax=(int *)realloc(xmax,s*s*sizeof(float));
data=(float **)realloc(data,s*s*sizeof(float *));
for (i=0; i<s*s; i++) data[i]=NULL;

}

void VolFrag::setSizeYZ(int nx, int y, int z) {
if (y>=size || y<0 || z>=size || z<0) return;
if (nx<0) nx=0;

setSizeI(nx,y+z*size);
}

void VolFrag::setSizeI(int nx, int i) {
if (i<0||i>size*size) return;
if (nx<0) nx=0;

xmax[i]=nx;
if (nx==0) { free(data[i]); data[i]=NULL; }
data[i]=(float *)realloc(data[i],nx*sizeof(float));
}



void VolFrag::zero() {
int i;

for (i=0; i<size*size; i++) {
	if (xmax[i]>0) memset(data[i],0,sizeof(float)*xmax[i]);	
}

}

// This will set up a fragment containing approximately 1 asymmetric unit
// of an icosahedron (actually a bit more than 1)
void VolFrag::setupIcos() {
int i,x,y,z;

for (y=-size/2,i=0; y<size/2; y++) {
	for (z=-size/2; z<size/2; z++,i++) {
		for (x=0; x<size/2; x++) {
			if (hypot3(x,y,z)>size*size/4) break;
			float axy = atan2(y,x);
			if (axy<PI/2-PI/2.5) break;
			float rxy=hypot(x,y);
			float alt=atan2(z,rxy);
			if (alt<42.29*PI/180.0) break;
		}
		if (x>0) setSizeI(x+1,i);		// expand by 1 to make sure we cover the space
		else setSizeI(0,i);
	}
}

}

void VolFrag::addFrag(VolFrag *src) {
if (src->getSize() != size) { printf("Different frag sizes\n"); return; }

for (int z=0; z<size; z++) {
	for (int y=0; y<size; y++) {
		for (int x=0; x<xmax[y+z*size]; x++) {
			addValue(x,y,z,src->valueAt(x,y,z));
		}
	}
}

}

// file contains size, xmax[size*size], data
int VolFrag::writeFile(char *file) {
int i;

FILE *out=fopen(file,"w");
if (!out) return -1;

fwrite(&size,sizeof(int),1,out);
fwrite(xmax,sizeof(int)*size,size,out);
for (i=0; i<size*size; i++) {
	if (xmax[i]>0) fwrite(data[i],sizeof(float),xmax[i],out);
}
fclose(out);

return 0;
}

int VolFrag::readFile(char *file) {
int i,j;

FILE *in=fopen(file,"r");
if (!in) return -1;

fread(&i,sizeof(int),1,in);
setSize(i);
fread(xmax,sizeof(int)*size,size,in);
for (i=0; i<size; i++) {
	for (j=0; j<size; j++) {
		setSizeYZ(xmax[j+i*size],j,i);
	}
}

for (i=0; i<size*size; i++) {
	if (xmax[i]>0) fread(data[i],sizeof(float),xmax[i],in);
}

return 0;
}
